import { AsyncDecoder, Decoder } from './Decoder';
import { DecoderError } from './DecoderResult';
export declare class AssertDecoderError extends Error {
  errors: DecoderError[];
  name: string;
  constructor(errors: DecoderError[]);
}
/**
 * When passed a decoder argument, `assert()` returns a new function which
 * can be used to decode the same values as the provided decoder. This new
 * function passes its input to the provided decoder's `decode()` method
 * and, on decode success, unwraps the processed value and returns it
 * directly. On failure, a `AssertDecoderError` is thrown
 * (rather than returned).
 *
 * ```ts
 * const validator = assert(numberD());
 * const value: number = validator(1);
 * const value: number = validator('1'); // will throw a `DecoderError`
 * ```
 */
export declare function assert<R, V>(
  decoder: Decoder<R, V>,
): {
  (value: Promise<V>): Promise<R>;
  (value: V): R;
};
export declare function assert<R, V>(
  decoder: AsyncDecoder<R, V>,
): (value: V | Promise<V>) => Promise<R>;
// # sourceMappingURL=assert.d.ts.map
