import { AsyncDecoder, Decoder, DecoderReturnType } from 'ts-decoders';
import { SimpleDecoderOptions } from './util';
export interface AnyOfDecoderOptions extends SimpleDecoderOptions {
  decodeInParallel?: boolean;
}
/**
 * Accepts an array of decoders and attempts to decode a provided value using
 * each of them, in order, returning the first successful result or `DecoderError[]`
 * if all fail. Unlike other decoder functions, `anyOfD()` always returns all
 * errors surfaced by it's decoder arguments. By default, decoder arguments are
 * tried in the order they are given.
 *
 * **Async:**
 * When calling `anyOfD()` with one or more `AsyncDecoder` arguments,
 * you can pass a `decodeInParallel: true` option to specify that a provided value
 * should be tried against all decoder arguments in parallel.
 */
export declare function anyOfD<T extends Decoder<any>>(
  decoders: T[],
  options?: AnyOfDecoderOptions,
): Decoder<DecoderReturnType<T>>;
export declare function anyOfD<T extends Decoder<any> | AsyncDecoder<any>>(
  decoders: T[],
  options?: AnyOfDecoderOptions,
): AsyncDecoder<DecoderReturnType<T>>;
// # sourceMappingURL=anyOf.d.ts.map
