import { AsyncDecoder, Decoder } from 'ts-decoders';
import { SimpleDecoderOptions } from './util';
export declare type ChainOfDecoderOptions = SimpleDecoderOptions;
/**
 * Accepts a spread or array of decoders and attempts to decode a
 * provided value using all of them, in order. The successful output
 * of one decoder is provided as input to the next decoder. `chainOfD()`
 * returns the `DecoderSuccess` value of the last decoder in the chain
 * or `DecoderError` on the first failure. Alternate names for this
 * decoder could be: pipe, compose, or allOf.
 *
 * Related:
 * - `Decoder#chain()`
 */
export declare function chainOfD<A, B, C, D, E, F, G>(
  a: Decoder<B, A>,
  b: Decoder<C, B>,
  c: Decoder<D, C>,
  d: Decoder<E, D>,
  e: Decoder<F, E>,
  f: Decoder<G, F>,
  options?: ChainOfDecoderOptions,
): Decoder<G, A>;
export declare function chainOfD<A, B, C, D, E, F>(
  a: Decoder<B, A>,
  b: Decoder<C, B>,
  c: Decoder<D, C>,
  d: Decoder<E, D>,
  e: Decoder<F, E>,
  options?: ChainOfDecoderOptions,
): Decoder<F, A>;
export declare function chainOfD<A, B, C, D, E>(
  a: Decoder<B, A>,
  b: Decoder<C, B>,
  c: Decoder<D, C>,
  d: Decoder<E, D>,
  options?: ChainOfDecoderOptions,
): Decoder<E, A>;
export declare function chainOfD<A, B, C, D>(
  a: Decoder<B, A>,
  b: Decoder<C, B>,
  c: Decoder<D, C>,
  options?: ChainOfDecoderOptions,
): Decoder<D, A>;
export declare function chainOfD<A, B, C>(
  a: Decoder<B, A>,
  b: Decoder<C, B>,
  options?: ChainOfDecoderOptions,
): Decoder<C, A>;
export declare function chainOfD<A, B>(
  a: Decoder<B, A>,
  options?: ChainOfDecoderOptions,
): Decoder<B, A>;
export declare function chainOfD<A, B = any>(
  decoders: [Decoder<any, A>, ...Array<Decoder<any>>],
  options?: ChainOfDecoderOptions,
): Decoder<B, A>;
export declare function chainOfD<A, B, C, D, E, F, G>(
  a: Decoder<B, A> | AsyncDecoder<B, A>,
  b: Decoder<C, B> | AsyncDecoder<C, B>,
  c: Decoder<D, C> | AsyncDecoder<D, C>,
  d: Decoder<E, D> | AsyncDecoder<E, D>,
  e: Decoder<F, E> | AsyncDecoder<F, E>,
  f: Decoder<G, F> | AsyncDecoder<G, F>,
  options?: ChainOfDecoderOptions,
): AsyncDecoder<G, A>;
export declare function chainOfD<A, B, C, D, E, F>(
  a: Decoder<B, A> | AsyncDecoder<B, A>,
  b: Decoder<C, B> | AsyncDecoder<C, B>,
  c: Decoder<D, C> | AsyncDecoder<D, C>,
  d: Decoder<E, D> | AsyncDecoder<E, D>,
  e: Decoder<F, E> | AsyncDecoder<F, E>,
  options?: ChainOfDecoderOptions,
): AsyncDecoder<F, A>;
export declare function chainOfD<A, B, C, D, E>(
  a: Decoder<B, A> | AsyncDecoder<B, A>,
  b: Decoder<C, B> | AsyncDecoder<C, B>,
  c: Decoder<D, C> | AsyncDecoder<D, C>,
  d: Decoder<E, D> | AsyncDecoder<E, D>,
  options?: ChainOfDecoderOptions,
): AsyncDecoder<E, A>;
export declare function chainOfD<A, B, C, D>(
  a: Decoder<B, A> | AsyncDecoder<B, A>,
  b: Decoder<C, B> | AsyncDecoder<C, B>,
  c: Decoder<D, C> | AsyncDecoder<D, C>,
  options?: ChainOfDecoderOptions,
): AsyncDecoder<D, A>;
export declare function chainOfD<A, B, C>(
  a: Decoder<B, A> | AsyncDecoder<B, A>,
  b: Decoder<C, B> | AsyncDecoder<C, B>,
  options?: ChainOfDecoderOptions,
): AsyncDecoder<C, A>;
export declare function chainOfD<A, B>(
  a: Decoder<B, A> | AsyncDecoder<B, A>,
  options?: ChainOfDecoderOptions,
): AsyncDecoder<B, A>;
export declare function chainOfD<A, B = any>(
  decoders: [
    Decoder<any, A> | AsyncDecoder<any, A>,
    ...Array<Decoder<any> | AsyncDecoder<any>>,
  ],
  options?: ChainOfDecoderOptions,
): AsyncDecoder<B, A>;
// # sourceMappingURL=chainOf.d.ts.map
