import { AsyncDecoder, Decoder } from 'ts-decoders';
import { ComposeDecoderOptions } from './util';
export declare type DictionaryDecoderOptions = ComposeDecoderOptions;
/**
 * Receives a decoder argument and uses that decoder to process all
 * values (regardless of key) of an input object. You can pass an
 * optional key decoder as the second argument which will be used to
 * decode each key of an input object.
 *
 * Options:
 * - If you pass an `allErrors: true` option as well as any
 * AsyncDecoders as arguments, then `dictionaryD()` will create a new
 * AsyncDecoder which decodes each key of the input object in parallel.
 *
 * Related:
 * - objectD
 * - arrayD
 */
export declare function dictionaryD<R, V>(
  valueDecoder: Decoder<R, V>,
  options?: DictionaryDecoderOptions,
): Decoder<
  {
    [key: string]: R;
  },
  V
>;
export declare function dictionaryD<R, V>(
  valueDecoder: Decoder<R, V>,
  keyDecoder: Decoder<string, string>,
  options?: DictionaryDecoderOptions,
): Decoder<
  {
    [key: string]: R;
  },
  V
>;
export declare function dictionaryD<R, V>(
  decoder: AsyncDecoder<R, V>,
  options?: DictionaryDecoderOptions,
): AsyncDecoder<
  {
    [key: string]: R;
  },
  V
>;
export declare function dictionaryD<R, V>(
  valueDecoder: Decoder<R, V> | AsyncDecoder<R, V>,
  keyDecoder: Decoder<string, string> | AsyncDecoder<string, string>,
  options?: DictionaryDecoderOptions,
): AsyncDecoder<
  {
    [key: string]: R;
  },
  V
>;
// # sourceMappingURL=dictionary.d.ts.map
