import { AsyncDecoder, Decoder } from 'ts-decoders';
import { ComposeDecoderOptions } from './util';
export interface ObjectDecoderOptions extends ComposeDecoderOptions {
  noExcessProperties?: boolean;
  removeUndefinedProperties?: boolean;
}
/**
 * Accepts a `{[key: string]: Decoder<any> | AsyncDecoder<any>}` init object
 * argument and returns a new decoder that will verify that an input is a
 * non-null object, and that each element-key of the input is decoded by the
 * corresponding element-key of the init object. On `DecoderSuccess`, a new
 * object is returned which has element-values defined by the init object's
 * element-values. By default, any excess properties on the input object are
 * ignored (i.e. not included on the returned value).
 *
 * - _"object element" refers to a `key: value` pair of the object._
 * - _"element-value" refers to the `value` of this pair._
 * - _"element-key" refers to the `key` of this pair_
 *
 * Options:
 * - If you pass the `noExcessProperties: true` option, any excess properties
 *   on the input object will return a DecoderError.
 * - If you pass an `allErrors: true` option as well as any AsyncDecoders as
 *   arguments, then `objectD()` will create a new AsyncDecoder which decodes
 *   each key of the input object in parallel.
 * - If you pass the `removeUndefinedProperties: true` option, then after all
 *   other decoding of an input succeeds, any `undefined` properties are
 *   deleted from the result.
 *
 * Related:
 * - dictionaryD
 */
export declare function objectD<T>(
  decoderObject: {
    [P in keyof T]: Decoder<T[P]>;
  },
  options?: ObjectDecoderOptions,
): Decoder<T>;
export declare function objectD<T>(
  decoderObject: {
    [P in keyof T]: Decoder<T[P]> | AsyncDecoder<T[P]>;
  },
  options?: ObjectDecoderOptions,
): AsyncDecoder<T>;
// # sourceMappingURL=object.d.ts.map
