import { AsyncDecoder, Decoder } from 'ts-decoders';
import { SimpleDecoderOptions } from './util';
export interface PredicateDecoderOptions extends SimpleDecoderOptions {
  promise?: boolean;
}
/**
 * Accepts a predicate function argument and creates a decoder which
 * verifies that inputs pass the function check.
 *
 * **Async**:
 * To pass a predicate function which returns a promise resolving to a
 * boolean, pass the `promise: true` option to `predicateD()`.
 */
export declare function predicateD<I>(
  fn: (value: I) => boolean | Promise<boolean>,
  options: PredicateDecoderOptions & {
    promise: true;
  },
): AsyncDecoder<I, I>;
export declare function predicateD<I, R extends I>(
  fn: (value: I) => value is R,
  options?: PredicateDecoderOptions,
): Decoder<R, I>;
export declare function predicateD<I>(
  fn: (value: I) => boolean,
  options?: PredicateDecoderOptions,
): Decoder<I, I>;
// # sourceMappingURL=predicate.d.ts.map
