import { AsyncDecoder, Decoder } from 'ts-decoders';
import { ComposeDecoderOptions } from './util';
export declare type TupleDecoderOptions = ComposeDecoderOptions;
/**
 * Receives an array of decoders and creates a decoder which
 * can be used to verify that an input is:
 *
 * 1. An array of the same length as the decoder argument array.
 * 2. The first decoder argument will be used the process the
 *    first element of an input array.
 * 3. The second decoder argument will be used the process the
 *    second element of an input array.
 * 4. etc...
 *
 * Options:
 * - If you pass an `allErrors: true` option as well as any
 *   AsyncDecoders as arguments, then `tupleD()` will create a new
 *   AsyncDecoder which decodes each index of the input array in
 *   parallel.
 *
 * Related:
 * - arrayD
 */
export declare function tupleD<R extends [any, ...any[]]>(
  decoders: {
    [P in keyof R]: Decoder<R[P]>;
  },
  options?: TupleDecoderOptions,
): Decoder<R>;
export declare function tupleD<R extends [any, ...any[]]>(
  decoders: {
    [P in keyof R]: Decoder<R[P]> | AsyncDecoder<R[P]>;
  },
  options?: TupleDecoderOptions,
): AsyncDecoder<R>;
// # sourceMappingURL=tuple.d.ts.map
