(function(global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined'
    ? factory(require('ts-decoders'))
    : typeof define === 'function' && define.amd
    ? define(['ts-decoders'], factory)
    : ((global = global || self), factory(global.tsDecoders));
})(this, function(tsDecoders) {
  'use strict';

  /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

  function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function(resolve, reject) {
      function fulfilled(value) {
        try {
          step(generator.next(value));
        } catch (e) {
          reject(e);
        }
      }
      function rejected(value) {
        try {
          step(generator['throw'](value));
        } catch (e) {
          reject(e);
        }
      }
      function step(result) {
        result.done
          ? resolve(result.value)
          : new P(function(resolve) {
              resolve(result.value);
            }).then(fulfilled, rejected);
      }
      step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
  }

  function __generator(thisArg, body) {
    var _ = {
        label: 0,
        sent: function() {
          if (t[0] & 1) throw t[1];
          return t[1];
        },
        trys: [],
        ops: [],
      },
      f,
      y,
      t,
      g;
    return (
      (g = { next: verb(0), throw: verb(1), return: verb(2) }),
      typeof Symbol === 'function' &&
        (g[Symbol.iterator] = function() {
          return this;
        }),
      g
    );
    function verb(n) {
      return function(v) {
        return step([n, v]);
      };
    }
    function step(op) {
      if (f) throw new TypeError('Generator is already executing.');
      while (_)
        try {
          if (
            ((f = 1),
            y &&
              (t =
                op[0] & 2
                  ? y['return']
                  : op[0]
                  ? y['throw'] || ((t = y['return']) && t.call(y), 0)
                  : y.next) &&
              !(t = t.call(y, op[1])).done)
          )
            return t;
          if (((y = 0), t)) op = [op[0] & 2, t.value];
          switch (op[0]) {
            case 0:
            case 1:
              t = op;
              break;
            case 4:
              _.label++;
              return { value: op[1], done: false };
            case 5:
              _.label++;
              y = op[1];
              op = [0];
              continue;
            case 7:
              op = _.ops.pop();
              _.trys.pop();
              continue;
            default:
              if (
                !((t = _.trys), (t = t.length > 0 && t[t.length - 1])) &&
                (op[0] === 6 || op[0] === 2)
              ) {
                _ = 0;
                continue;
              }
              if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                _.label = op[1];
                break;
              }
              if (op[0] === 6 && _.label < t[1]) {
                _.label = t[1];
                t = op;
                break;
              }
              if (t && _.label < t[2]) {
                _.label = t[2];
                _.ops.push(op);
                break;
              }
              if (t[2]) _.ops.pop();
              _.trys.pop();
              continue;
          }
          op = body.call(thisArg, _);
        } catch (e) {
          op = [6, e];
          y = 0;
        } finally {
          f = t = 0;
        }
      if (op[0] & 5) throw op[1];
      return { value: op[0] ? op[1] : void 0, done: true };
    }
  }

  function printValue(value) {
    return value instanceof Promise ? value : JSON.stringify(value);
  }
  /**
   * Make an assertion that `actual` is a Decoder.
   *
   * If not then throw.
   */
  expect.extend({
    toBeDecoder: function(received) {
      return {
        message: function() {
          return '"' + printValue(received) + '" expected to be a Decoder';
        },
        pass: received instanceof tsDecoders.Decoder,
      };
    },
  });
  /**
   * Make an assertion that `actual` is a AsyncDecoder.
   *
   * If not then throw.
   */
  expect.extend({
    toBeAsyncDecoder: function(received) {
      return {
        message: function() {
          return '"' + printValue(received) + '" expected to be a AsyncDecoder';
        },
        pass: received instanceof tsDecoders.AsyncDecoder,
      };
    },
  });
  /**
   * Make an assertion that `actual` is a DecoderSuccess.
   *
   * If not then throw.
   */
  expect.extend({
    toBeDecoderSuccess: function(received, expected) {
      var pass = !(received instanceof Promise);
      if (!pass) {
        return {
          message: function() {
            return (
              '"' + printValue(received) + '" expected to not be a promise'
            );
          },
          pass: false,
        };
      }
      expect(received).toEqual(expected);
      return {
        message:
          '"' +
          printValue(received) +
          '" expected to equal ' +
          printValue(expected),
        pass: true,
      };
    },
  });
  /**
   * Make an assertion that `actual` is a DecoderSuccess
   * with matching `value` to `expected`.
   *
   * If not then throw.
   */
  expect.extend({
    toBeAsyncDecoderSuccess: function(received, expected) {
      return __awaiter(this, void 0, void 0, function() {
        var pass, value;
        return __generator(this, function(_a) {
          switch (_a.label) {
            case 0:
              pass = received instanceof Promise;
              if (!pass) {
                return [
                  2 /*return*/,
                  {
                    message: function() {
                      return (
                        '"' +
                        printValue(received) +
                        '" expected to be a promise'
                      );
                    },
                    pass: false,
                  },
                ];
              }
              return [4 /*yield*/, received];
            case 1:
              value = _a.sent();
              expect(value).toEqual(expected);
              return [
                2 /*return*/,
                {
                  message:
                    '"' +
                    printValue(value) +
                    '" expected to equal ' +
                    printValue(expected),
                  pass: true,
                },
              ];
          }
        });
      });
    },
  });
  /**
   * Make an assertion that `actual` is an array of DecoderError
   * with matching properties and size to `expected`. Also asserts that
   * `actual` is NOT an instanceof `Promise`.
   *
   * If not then throw.
   */
  expect.extend({
    toBeDecoderErrors: function(received, expected) {
      if (received instanceof Promise) {
        return {
          message: function() {
            return (
              '"' + printValue(received) + '" expected to not be a promise'
            );
          },
          pass: false,
        };
      }
      if (
        !Array.isArray(received) ||
        received.some(function(val) {
          return !(val instanceof tsDecoders.DecoderError);
        })
      ) {
        return {
          message: function() {
            return (
              printValue(received) + ' expected to be an array of DecoderError'
            );
          },
          pass: false,
        };
      }
      var normExpected =
        expected instanceof tsDecoders.DecoderError ? [expected] : expected;
      if (received.length !== normExpected.length) {
        return {
          message: function() {
            return (
              printValue(received) +
              ' expected to have length ' +
              normExpected.length +
              ' but had length ' +
              received.length
            );
          },
          pass: false,
        };
      }
      normExpected.forEach(function(value) {
        expect(received).toContainEqual(value);
      });
      return {
        message: printValue(received) + ' expected to contain items',
        pass: true,
      };
    },
  });
  /**
   * Similar to `assertDecoderErrors`, except also asserts that
   * `actual` is an instanceof `Promise`.
   */
  expect.extend({
    toBeAsyncDecoderErrors: function(received, expected) {
      return __awaiter(this, void 0, void 0, function() {
        var value;
        return __generator(this, function(_a) {
          switch (_a.label) {
            case 0:
              if (!(received instanceof Promise)) {
                return [
                  2 /*return*/,
                  {
                    message: function() {
                      return (
                        '"' +
                        printValue(received) +
                        '" expected to be a promise'
                      );
                    },
                    pass: false,
                  },
                ];
              }
              return [4 /*yield*/, received];
            case 1:
              value = _a.sent();
              expect(value).toBeDecoderErrors(expected);
              return [
                2 /*return*/,
                {
                  message:
                    printValue(value) +
                    ' expected to be an array of decoder errors',
                  pass: true,
                },
              ];
          }
        });
      });
    },
  });
  /**
   * Make an assertion that `decoder.decode(value)` resolves to a
   * DecoderSuccess with matching `value` to `expected`.
   */
  expect.extend({
    toDecodeSuccessfully: function(_a, expected) {
      var decoder = _a[0],
        value = _a[1];
      expect(decoder.decode(value)).toBeDecoderSuccess(expected);
      return {
        message: printValue(value) + ' expected to decode successfully',
        pass: true,
      };
    },
  });
  expect.extend({
    toAsyncDecodeSuccessfully: function(_a, expected) {
      var decoder = _a[0],
        value = _a[1];
      return __awaiter(this, void 0, void 0, function() {
        return __generator(this, function(_b) {
          switch (_b.label) {
            case 0:
              if (
                !(
                  decoder instanceof tsDecoders.AsyncDecoder ||
                  value instanceof Promise
                )
              ) {
                return [
                  2 /*return*/,
                  {
                    message: function() {
                      return (
                        (decoder && decoder.constructor.name) +
                        ' expected to be a AsyncDecoder'
                      );
                    },
                    pass: false,
                  },
                ];
              }
              return [
                4 /*yield*/,
                expect(decoder.decode(value)).toBeAsyncDecoderSuccess(expected),
              ];
            case 1:
              _b.sent();
              return [
                2 /*return*/,
                {
                  message:
                    printValue(value) +
                    ' expected to async decode successfully',
                  pass: true,
                },
              ];
          }
        });
      });
    },
  });
  /**
   * Make an assertion that `decoder.decode(value)` resolves to an
   * array of DecoderError with matching properties and size to
   * `expected`.
   *
   * If not then throw.
   */
  expect.extend({
    toDecodeWithErrors: function(_a, expected) {
      var decoder = _a[0],
        value = _a[1];
      expect(decoder.decode(value)).toBeDecoderErrors(expected);
      return {
        message: printValue(value) + ' expected to decode with errors',
        pass: true,
      };
    },
  });
  expect.extend({
    toAsyncDecodeWithErrors: function(_a, expected) {
      var decoder = _a[0],
        value = _a[1];
      return __awaiter(this, void 0, void 0, function() {
        return __generator(this, function(_b) {
          switch (_b.label) {
            case 0:
              if (
                !(
                  decoder instanceof tsDecoders.AsyncDecoder ||
                  value instanceof Promise
                )
              ) {
                return [
                  2 /*return*/,
                  {
                    message: function() {
                      return (
                        decoder.constructor.name +
                        ' expected to be a AsyncDecoder'
                      );
                    },
                    pass: false,
                  },
                ];
              }
              return [
                4 /*yield*/,
                expect(decoder.decode(value)).toBeAsyncDecoderErrors(expected),
              ];
            case 1:
              _b.sent();
              return [
                2 /*return*/,
                {
                  message:
                    printValue(value) + ' expected to async decode with errors',
                  pass: true,
                },
              ];
          }
        });
      });
    },
  });
});
