(function(global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined'
    ? factory(exports, require('ts-decoders'))
    : typeof define === 'function' && define.amd
    ? define(['exports', 'ts-decoders'], factory)
    : ((global = global || self),
      factory((global.tsDecodersTesting = {}), global.tsDecoders));
})(this, function(exports, tsDecoders) {
  'use strict';

  /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

  function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function(resolve, reject) {
      function fulfilled(value) {
        try {
          step(generator.next(value));
        } catch (e) {
          reject(e);
        }
      }
      function rejected(value) {
        try {
          step(generator['throw'](value));
        } catch (e) {
          reject(e);
        }
      }
      function step(result) {
        result.done
          ? resolve(result.value)
          : new P(function(resolve) {
              resolve(result.value);
            }).then(fulfilled, rejected);
      }
      step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
  }

  function __generator(thisArg, body) {
    var _ = {
        label: 0,
        sent: function() {
          if (t[0] & 1) throw t[1];
          return t[1];
        },
        trys: [],
        ops: [],
      },
      f,
      y,
      t,
      g;
    return (
      (g = { next: verb(0), throw: verb(1), return: verb(2) }),
      typeof Symbol === 'function' &&
        (g[Symbol.iterator] = function() {
          return this;
        }),
      g
    );
    function verb(n) {
      return function(v) {
        return step([n, v]);
      };
    }
    function step(op) {
      if (f) throw new TypeError('Generator is already executing.');
      while (_)
        try {
          if (
            ((f = 1),
            y &&
              (t =
                op[0] & 2
                  ? y['return']
                  : op[0]
                  ? y['throw'] || ((t = y['return']) && t.call(y), 0)
                  : y.next) &&
              !(t = t.call(y, op[1])).done)
          )
            return t;
          if (((y = 0), t)) op = [op[0] & 2, t.value];
          switch (op[0]) {
            case 0:
            case 1:
              t = op;
              break;
            case 4:
              _.label++;
              return { value: op[1], done: false };
            case 5:
              _.label++;
              y = op[1];
              op = [0];
              continue;
            case 7:
              op = _.ops.pop();
              _.trys.pop();
              continue;
            default:
              if (
                !((t = _.trys), (t = t.length > 0 && t[t.length - 1])) &&
                (op[0] === 6 || op[0] === 2)
              ) {
                _ = 0;
                continue;
              }
              if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                _.label = op[1];
                break;
              }
              if (op[0] === 6 && _.label < t[1]) {
                _.label = t[1];
                t = op;
                break;
              }
              if (t && _.label < t[2]) {
                _.label = t[2];
                _.ops.push(op);
                break;
              }
              if (t[2]) _.ops.pop();
              _.trys.pop();
              continue;
          }
          op = body.call(thisArg, _);
        } catch (e) {
          op = [6, e];
          y = 0;
        } finally {
          f = t = 0;
        }
      if (op[0] & 5) throw op[1];
      return { value: op[0] ? op[1] : void 0, done: true };
    }
  }

  var _this = undefined;
  /**
   * Test Decoders
   */
  /** Simple boolean Decoder for use in tests */
  var BOOLEAN_DEC = new tsDecoders.Decoder(function(value) {
    return typeof value === 'boolean'
      ? new tsDecoders.DecoderSuccess(value)
      : new tsDecoders.DecoderError(value, 'invalid type', 'must be a boolean');
  });
  /** Simple string Decoder for use in tests */
  var STRING_DEC = new tsDecoders.Decoder(function(value) {
    return typeof value === 'string'
      ? new tsDecoders.DecoderSuccess(value)
      : new tsDecoders.DecoderError(value, 'invalid type', 'must be a string');
  });
  /** Simple number Decoder for use in tests */
  var NUMBER_DEC = new tsDecoders.Decoder(function(value) {
    return typeof value === 'number'
      ? new tsDecoders.DecoderSuccess(value)
      : new tsDecoders.DecoderError(value, 'invalid type', 'must be a number');
  });
  /** Simple `any` Decoder for use in tests */
  var ANY_DEC = new tsDecoders.Decoder(function(value) {
    return new tsDecoders.DecoderSuccess(value);
  });
  /** Recursive "nested arrays" Decoder for use in tests */
  var RECURSIVE_DEC = new tsDecoders.Decoder(function(input) {
    if (!Array.isArray(input)) {
      return new tsDecoders.DecoderError(
        input,
        'invalid type',
        'must be an array',
      );
    }
    var index = -1;
    var array = [];
    var _loop_1 = function(element) {
      index++;
      var result = RECURSIVE_DEC.decode(element);
      if (tsDecoders.areDecoderErrors(result)) {
        return {
          value: result.map(function(error) {
            return new tsDecoders.DecoderError(
              element,
              'invalid element',
              'invalid element [' + index + '] > ' + error.message,
              {
                location: '[' + index + ']' + error.location,
                key: index,
                child: error,
              },
            );
          }),
        };
      }
      array.push(result.value);
    };
    for (var _i = 0, input_1 = input; _i < input_1.length; _i++) {
      var element = input_1[_i];
      var state_1 = _loop_1(element);
      if (typeof state_1 === 'object') return state_1.value;
    }
    return new tsDecoders.DecoderSuccess(array);
  });
  /**
   * Test AsyncDecoders
   */
  /** Simple boolean AsyncDecoder for use in tests */
  var BOOLEAN_ASYNC_DEC = new tsDecoders.AsyncDecoder(function(value) {
    return __awaiter(_this, void 0, void 0, function() {
      return __generator(this, function(_a) {
        return [
          2 /*return*/,
          typeof value === 'boolean'
            ? new tsDecoders.DecoderSuccess(value)
            : new tsDecoders.DecoderError(
                value,
                'invalid type',
                'must be a boolean',
              ),
        ];
      });
    });
  });
  /** Simple number AsyncDecoder for use in tests */
  var STRING_ASYNC_DEC = new tsDecoders.AsyncDecoder(function(value) {
    return __awaiter(_this, void 0, void 0, function() {
      return __generator(this, function(_a) {
        return [
          2 /*return*/,
          typeof value === 'string'
            ? new tsDecoders.DecoderSuccess(value)
            : new tsDecoders.DecoderError(
                value,
                'invalid type',
                'must be a string',
              ),
        ];
      });
    });
  });
  /** Simple number AsyncDecoder for use in tests */
  var NUMBER_ASYNC_DEC = new tsDecoders.AsyncDecoder(function(value) {
    return __awaiter(_this, void 0, void 0, function() {
      return __generator(this, function(_a) {
        return [
          2 /*return*/,
          typeof value === 'number'
            ? new tsDecoders.DecoderSuccess(value)
            : new tsDecoders.DecoderError(
                value,
                'invalid type',
                'must be a number',
              ),
        ];
      });
    });
  });
  /** Simple number AsyncDecoder for use in tests */
  var ANY_ASYNC_DEC = new tsDecoders.AsyncDecoder(function(value) {
    return __awaiter(_this, void 0, void 0, function() {
      return __generator(this, function(_a) {
        return [2 /*return*/, new tsDecoders.DecoderSuccess(value)];
      });
    });
  });
  /** Recursive "nested array" AsyncDecoder for use in tests */
  var RECURSIVE_ASYNC_DEC = new tsDecoders.AsyncDecoder(function(input) {
    return __awaiter(_this, void 0, void 0, function() {
      var index, array, _loop_2, _i, input_2, element, state_2;
      return __generator(this, function(_a) {
        switch (_a.label) {
          case 0:
            if (!Array.isArray(input)) {
              return [
                2 /*return*/,
                new tsDecoders.DecoderError(
                  input,
                  'invalid type',
                  'must be an array',
                ),
              ];
            }
            index = -1;
            array = [];
            _loop_2 = function(element) {
              var result;
              return __generator(this, function(_a) {
                switch (_a.label) {
                  case 0:
                    index++;
                    return [4 /*yield*/, RECURSIVE_ASYNC_DEC.decode(element)];
                  case 1:
                    result = _a.sent();
                    if (tsDecoders.areDecoderErrors(result)) {
                      return [
                        2 /*return*/,
                        {
                          value: result.map(function(error) {
                            return new tsDecoders.DecoderError(
                              element,
                              'invalid element',
                              'invalid element [' +
                                index +
                                '] > ' +
                                error.message,
                              {
                                location: '[' + index + ']' + error.location,
                                key: index,
                                child: error,
                              },
                            );
                          }),
                        },
                      ];
                    }
                    array.push(result.value);
                    return [2 /*return*/];
                }
              });
            };
            (_i = 0), (input_2 = input);
            _a.label = 1;
          case 1:
            if (!(_i < input_2.length)) return [3 /*break*/, 4];
            element = input_2[_i];
            return [5 /*yield**/, _loop_2(element)];
          case 2:
            state_2 = _a.sent();
            if (typeof state_2 === 'object')
              return [2 /*return*/, state_2.value];
            _a.label = 3;
          case 3:
            _i++;
            return [3 /*break*/, 1];
          case 4:
            return [2 /*return*/, new tsDecoders.DecoderSuccess(array)];
        }
      });
    });
  });

  exports.ANY_ASYNC_DEC = ANY_ASYNC_DEC;
  exports.ANY_DEC = ANY_DEC;
  exports.BOOLEAN_ASYNC_DEC = BOOLEAN_ASYNC_DEC;
  exports.BOOLEAN_DEC = BOOLEAN_DEC;
  exports.NUMBER_ASYNC_DEC = NUMBER_ASYNC_DEC;
  exports.NUMBER_DEC = NUMBER_DEC;
  exports.RECURSIVE_ASYNC_DEC = RECURSIVE_ASYNC_DEC;
  exports.RECURSIVE_DEC = RECURSIVE_DEC;
  exports.STRING_ASYNC_DEC = STRING_ASYNC_DEC;
  exports.STRING_DEC = STRING_DEC;

  Object.defineProperty(exports, '__esModule', { value: true });
});
